﻿var sudoku;
var counter;
var helpTimer1;
var helpTimer2;
var webWorker;
var helpWait = 30000;
var maxMoves = 100;
var currentUser;
var currentUserData;
var nodeServer = "http://localhost:8080/";

$(document).ready(function() {
	// Umschalten zwischen Spiel und Dokumentation
	$("#switch").click(function(){
		$(".content-page").toggle("hidden");
		if ($("#switch").html() == "Spielen")
			$("#switch").html("Dokumentation");
		else
			$("#switch").html("Spielen");
	});
	// Dynamischer Aufbau des Spielfelds
	var gameTable = $("#game");
	function getCellID(blockrow, blockcol, row, col){
		return ((blockrow-1)*3 + row) + "-" + ((blockcol-1)*3 + col);
	}
	for (var blockrow = 1; blockrow <= 3; blockrow++){
		var bRow = $("<tr>");
		for (var blockcol = 1; blockcol <= 3; blockcol++){
			var bCol = $("<td id='"+"b"+blockrow+"-"+blockcol+"'>");
			var bTable = $("<table>");
			for (var row = 1; row <= 3; row++){
				var cRow = $("<tr>");
				for (var col = 1; col <= 3; col++){
					var cCol = $("<td>");
					cCol.append($("<input id='"+getCellID(blockrow, blockcol, row, col)+"' class='field' type='text'>"));
					cRow.append(cCol);
				}
				bTable.append(cRow);
			}
			bCol.append(bTable);
			bRow.append(bCol);
		}
		gameTable.append(bRow);
	}
	// "Aktiv"-Markierung
	$(".field").click(function(){
		$(".field").removeClass("active");
		$(this).addClass("active");
	});
	function onSolved(){
		stopTimer();
		$(".field").addClass("active");
		$("#game").addClass("solved");
		$("#start").focus();
		saveScore();
	}
	function onHelp(){
		sudoku.markHelp();
		helpTimer2 = setTimeout(function(){
			if (sudoku.solveHelp())
				onSolved();
			else
				helpTimer1 = setTimeout(onHelp,helpWait);
		},helpWait);
	}
	function startTimer(){
		webWorker = new Worker("Worker.js");
		webWorker.onmessage = function(e) {
			counter = e.data;
			$("#counter").html(counter);
		}
		counter = 0;
		webWorker.postMessage("Start");
	}
	function stopTimer(){
		if (webWorker)	
			webWorker.postMessage("Stop");
		webWorker = undefined;
	}
	// Eingabevalidierung
    $(".field").on("input", function(){
		if ($(this).val().match("[1-9]") != $(this).val())
			$(this).val("");
		$(".field").removeClass("helpmark");
		clearTimeout(helpTimer1);
		clearTimeout(helpTimer2);
		if (sudoku.set(this.id, $(this).val()))
			onSolved();
		else
			helpTimer1 = setTimeout(onHelp,helpWait);
	});
	// Startbelegung
	$("#start").click(function(){
		sudoku = new Sudoku();
		sudoku.create(maxMoves);
		sudoku.show();
		$("#game").removeClass("solved");
		$(".field").removeClass("active");
		$(".field").removeClass("helpmark");
		clearTimeout(helpTimer1);
		clearTimeout(helpTimer2);
		stopTimer();
		startTimer();
		helpTimer1 = setTimeout(onHelp,helpWait);
	});
	// Login und Logout
	currentUser = "";
	$("#loginout").click(function(){
		if (currentUser == ""){ // login
			$("#username").prop('disabled', true);
			$("#password").prop('disabled', true);
			$("#loginout").prop('disabled', true);
			var data = {
				username: $("#username").val(),
				password: $("#password").val()
			};
			$.post(nodeServer+"login",data, function(res){
				if(res.state === 'OK')
				{
					currentUser = res.user;
					currentUserData = res.data;
					updateScores();
					$("#loginout img").attr("src", "exit.png");
				}
				else{
					$("#username").prop('disabled', false);
					$("#password").prop('disabled', false);
				}
				$("#loginout").prop('disabled', false);
			}).fail(function(xhr, status, error) {
				$("#username").prop('disabled', false);
				$("#password").prop('disabled', false);
			});
		}
		else { // logout
			currentUser = "";
			currentUserData = undefined;
			updateScores();
			$("#username").prop('disabled', false);
			$("#password").prop('disabled', false);
			$("#username").val("");
			$("#password").val("");
			$("#loginout img").attr("src","login.png");
		}
	});
	function saveScore(){
		if (currentUser == "") return;
		var data = {
			username: $("#username").val(),
			score: counter,
			date: new Date()
		};
		$.post(nodeServer+"savescore",data, function(res){
			if(res.state === 'OK')
			{
				currentUser = res.user;
				currentUserData = res.data;
				updateScores();
			}
		});
	}
	function updateScores(){
		$("#scores").empty();
		if (currentUserData == undefined) return;
		if (currentUserData.score == undefined) return;
		currentUserData.score.sort(function(a,b){return a.time-b.time;});
		for (var i = 0; i < currentUserData.score.length; i++){
			var s = currentUserData.score[i];
			var line = new Date(s.date).toLocaleString();
			line += ": " + s.time;
			$("#scores").append($("<li>").html(line));
			if (i == 9) break;
		}
	}
});
